<?php

/** @file
 * Extra stuffs for privileged user
 * By jcisio
 */

/**
 * Generic menu callback for select/deselect/change an answer.
 *
 * @param $action, $cid, $token
 *
 */
function question_answer_action($action, $cid, $token) {
  if (is_numeric($cid) && drupal_valid_token($token, "question_answer/$action/$cid")) {
    $nid = db_result(db_query("SELECT nid FROM {comments} WHERE cid = %d", $cid));
    if (is_numeric($nid)) {
      $node = node_load($nid);
      $qtype = _question_answer_nodetypes();

      if ($node) {
        // If we are here, the token is valid, it means the action is good to be
        // done. No other check is needed.
        $node->{$qtype[$node->type]['answer']}[0]['value'] = ($action == 'deselect') ? NULL : $cid;
        node_save($node);

        // Let's add some feedback to the user
        $message = ($action == 'deselect') ? t('The previous choice for the <em>best answer</em> (comment #@cid) is now cancelled. This question is once again available for selecting the best answer.', array('@cid' => $cid)) : t('The comment #@cid is now selected as the current <em>best answer</em>.', array('@cid' => $cid));
        drupal_set_message($message, 'status');
      }
    }
  }

  if (! $node) {
    drupal_set_message(t('Oops! There was an unknown error. Question not found'), 'warning');
  }

  if (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest') {
    ctools_include('ajax');
    $commands = array();
    if ($node) {
      // It is complicated to update page elements (for selected answer just
      // below node content and in the comment list, links for each comment to
      // be added/removed). Therefore the best solution is to reload the current
      // page.
      $commands[] = ctools_ajax_command_reload();
    }
    ctools_ajax_render($commands);
  }
  else {
    drupal_goto();
  }
}

/**
 * Settings form
 */
function question_answer_settings() {
  $form = array();

  $nodes = array();
  $fields = array();
  foreach (content_fields() as $field) {
    $fields[$field['field_name']] = $field['field_name'];
    if (! in_array($field['type_name'], $nodes)) {
      $nodes[$field['type_name']] = $field['type_name'];
    }
  }

  $node_types = _question_answer_nodetypes();
  $node_types_keys = array_keys($node_types);
  $node_types_values = array_values($node_types);
  $nodetype = array_shift($node_types_keys);
  $options = array_shift($node_types_values);

  $form['question_answer_help'] = array(
    '#value' => '<p>' . t('A content type is used as question. Comments in nodes of this type is considered answers. This node type should have appropriately configured fields to store the answer (comment ID), the duration etc. If you don\'t have a content type or you haven\'t created required fields, please do it in the <a href="!link">Content types</a> settings page.', array('!link' => url('admin/content/types'))) . '</p>',
  );
  $form['question_answer_nodetype'] = array(
    '#type' => 'select',
    '#title' => t('Node type for questions'),
    '#description' => t('Select the node type you want to use for the Question/Answer feature. Only node types with appropriately configured fields are available.'),
    '#required' => TRUE,
    '#options' => $nodes,
    '#default_value' => $nodetype,
  );

  $form['question_answer_fieldtype'] = array(
    '#type' => 'select',
    '#title' => t('Field type for answer'),
    '#description' => t('This must be an <em>Integer</em> and mono value field. This field is used to store the ID of selected answered (comment).'),
    '#required' => TRUE,
    '#options' => $fields,
    '#default_value' => $options['answer'],
  );

  $form['question_answer_duration'] = array(
    '#type' => 'select',
    '#title' => t('Field type for duration'),
    '#description' => t('This must be a <em>Integer</em> mono value field using the <em>Select list</em> widget. This field contains a list of pre-configured time limits for the question. The used unit is second. You must display this field using the <em>Remaining time</em> format and have VotingAPI enabled if you want to use the expiration feature. For more information, please read <a href="!link">the documentation</a>.', array('!link' => 'http://drupal.org/node/955460')),
    '#required' => FALSE,
    '#options' => array('' => t('Please select a field')) + $fields,
    '#default_value' => $options['duration'],
  );

  $form['question_answer_maxlength'] = array(
    '#type' => 'textfield',
    '#title' => t('Node title maxlength'),
    '#required' => TRUE,
    '#default_value' => variable_get('question_answer_maxlength', 0),
    '#description' => t('Maxlength of the question title. Default is 0 (unlimited)'),
  );

  $form['question_answer_toplevel_comments'] = array(
    '#type' => 'select',
    '#title' => t('Top level comments only'),
    '#required' => FALSE,
    '#default_value' => variable_get('question_answer_toplevel_comments', 0),
    '#options' => array(
      '1' => t('Yes'),
      '0' => t('No'),
    ),
    '#description' => t('Choose "Yes" if you want only <em>top level comments</em> to be available for being selected as <em>best answer</em>. The default behavior is "No", it means that all comments can be selected as <em>best answer</em>. Change to this option is not retrospective.'),
  );

  return system_settings_form($form);
}
